package com.engine.sskj.util;

import com.engine.kq.biz.KQOvertimeRulesBiz;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.formmode.setup.ModeRightInfo;
import weaver.general.BaseBean;
import weaver.general.Util;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author chenwnj
 * @date 2023/10/20
 * @description
 **/
public class SskjUtil {

    BaseBean basebean = new BaseBean();

    final String THISMONTH = "uf_jbtxdyfjs";
    final String OTHERMONTH = "uf_jbtxgyfff";


    /**
     * 建模权限重构
     * @param userId
     * @param modeId
     * @param billid
     */
    public void modePerRecon(Integer userId, String modeId, String billid) {
        ModeRightInfo ModeRightInfo = new ModeRightInfo();
        ModeRightInfo.setNewRight(true);
        ModeRightInfo.editModeDataShare( userId, Integer.parseInt(modeId), Integer.parseInt(billid));
    }

    /**
     * 建模权限重构
     * @param billid_creator
     * @param modeId
     * @param billids
     */
    public void modePerReconBatch(Map<Integer,Integer> billid_creator, String modeId, List<Integer> billids) {
        ModeRightInfo ModeRightInfo = new ModeRightInfo();
        ModeRightInfo.setNewRight(true);
        ModeRightInfo.editModeDataShare( billid_creator, Integer.parseInt(modeId), billids);
    }

    /**
     * 获取两个时间相差分钟数
     */
    public long getTime(String oldTime,String newTime) {

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        long diff = 0;
        try {
            long NTime =df.parse(newTime).getTime();
            //从对象中拿到时间
            long OTime = df.parse(oldTime).getTime();
            diff=(NTime-OTime)/1000/60;
        }catch (Exception e) {
            basebean.writeLog("SskjUtil getTime Exception: " + e);
        }

        return diff;
    }

    /**
     * 判断培训时长扣减去哪张表中查找
     */
    public String getTableName(String resourceId, String kqDate) {
        basebean.writeLog("SskjUtil getTableName start");

        RecordSet rs = new RecordSet();
        String tableName = "";
        //首先判断该人员是当月还是隔月  4-当月，5、6-隔月
        String acqOverTimeTypeSql = "select field28 from cus_fielddata where id = ? and scopeid = 3 and scope = 'HrmCustomFieldByInfoType'";
        rs.executeQuery(acqOverTimeTypeSql, resourceId);
        while ( rs.next()) {
            String field28 = Util.null2String(rs.getString("field28"));
            if (StringUtils.isNotBlank(field28)) {
                if ("4".equals(field28)) {
                    tableName = THISMONTH;
                }
                if ("5".equals(field28) || "6".equals(field28)) {
                    tableName = OTHERMONTH;
                }
            }
        }

        //再判断找那个明细表
        if (StringUtils.isNotBlank(tableName)) {
            //根据加班日期的类型来判断
            //1-节假日、2-工作日、3-休息日、-1-数据异常，无效数据
            int changeType = KQOvertimeRulesBiz.getChangeType(resourceId, kqDate);
            if (changeType == 1) {
                tableName = tableName + "_dt1";
            } else if (changeType == 2) {
                tableName = tableName + "_dt3";
            } else if (changeType == 3) {
                tableName = tableName + "_dt4";
            }
        }

        return tableName;

    }

    /**
     * 根据加班时长处理是否需要删除转事假对应的事假
     * @param resourceId 加班人
     * @param kqDate 加班的归属日期
     * @param requestid 加班流程请求id
     * @param duration 加班时长
     */
    public boolean handleCasLeave(String resourceId, String kqDate, String requestid, double duration) {
        basebean.writeLog("SskjUtil handleCasLeave start");

        String tableName = getTableName(resourceId, kqDate);
        basebean.writeLog("tableName: " + tableName);

        String[] split = tableName.split("_dt");
        String handleTabel = split[0] + "_dt6";
        basebean.writeLog("handleTabel: " + handleTabel);

        String acqCasData = "select qjsc, qjbid from " + handleTabel + " where sfdk = ? and xm = ? and ny = ?";
        basebean.writeLog("acqCasData: " + acqCasData);

        RecordSet rs = new RecordSet();

        //获取已抵扣转事假的时长
        //加班类型:1-节假日、2-工作日、3-休息日、-1-数据异常，无效数据
        int changeType = KQOvertimeRulesBiz.getChangeType(resourceId, kqDate);
        basebean.writeLog("changeType: " + changeType);

        double tempDecu = 0.00;
        if (changeType == 1) {
            String acqToCasSql = "select jrjbsy from " + split[0] + " where xm = ? and ny = ?";
            basebean.writeLog("acqToCasSql: " + acqToCasSql);

            rs.executeQuery(acqToCasSql, resourceId, kqDate.substring(0,7));
            while ( rs.next()) {
                tempDecu = Util.getDoubleValue(Util.null2String(rs.getString("jrjbsy")));
            }
            basebean.writeLog("tempDecu: " + tempDecu);

        } else if (changeType == 2) {
            String acqToCasSql = "select psjbsy from " + split[0] + " where xm = ? and ny = ?";
            basebean.writeLog("acqToCasSql: " + acqToCasSql);

            rs.executeQuery(acqToCasSql, resourceId, kqDate.substring(0,7));
            while ( rs.next()) {
                tempDecu = Util.getDoubleValue(Util.null2String(rs.getString("psjbsy")));
            }
            basebean.writeLog("tempDecu: " + tempDecu);
        } else if (changeType == 3) {
            String acqToCasSql = "select zmjbsy from " + split[0] + " where xm = ? and ny = ?";
            basebean.writeLog("acqToCasSql: " + acqToCasSql);

            rs.executeQuery(acqToCasSql, resourceId, kqDate.substring(0,7));
            while ( rs.next()) {
                tempDecu = Util.getDoubleValue(Util.null2String(rs.getString("zmjbsy")));
            }
            basebean.writeLog("tempDecu: " + tempDecu);
        }

        Map<String, Double> casMap = new HashMap<>();
        rs.executeQuery(acqCasData, resourceId, kqDate.substring(0,7));
        while ( rs.next()) {
            double qjsc = Util.getDoubleValue(Util.null2String(rs.getString("qjsc")));
            String qjbid = Util.null2String(rs.getString("qjbid"));
            if (StringUtils.isNotBlank(qjbid)) {
                casMap.put("qjbid", qjsc);
            }
        }
        if (casMap != null && casMap.size() > 0) {
            String modCasSql = "update " + handleTabel + " set qjsc = ?, sfdk = ?, dklcid = ? where qjbid = ?";
            basebean.writeLog("modCasSql: " + modCasSql);

            String modLeaveSql = "update uf_casleave set duration = ?, isdecu = ?, decuflowid = ? where id = ?";
            basebean.writeLog("modLeaveSql: " + modLeaveSql);

            String backField = "";
            if (changeType == 1) {
                backField = "jrjbsy";
            } else if (changeType == 2) {
                backField = "psjbsy";
            } else if (changeType == 3) {
                backField = "zmjbsy";
            }
            basebean.writeLog("backField: " + backField);

            String modDecuSql = "update "+ split[0] +" set " + backField + " = ? where xm = ? and ny = ?";
            basebean.writeLog("modDecuSql: " + modDecuSql);

            List<List> modCasList = new ArrayList<>();
            List tempList = new ArrayList();
            for (String key : casMap.keySet()) {
                double qjsc = casMap.get(key);
                double temp = duration - qjsc;
                if (temp >= 0) {
                    tempList.add(0.00);
                    tempList.add(1);
                    tempList.add(key);
                    tempList.add(requestid);
                    modCasList.add(tempList);
                    tempDecu = tempDecu + qjsc;
                } else {
                    tempList.add(Math.abs(temp));
                    tempList.add(0);
                    tempList.add(key);
                    tempList.add(requestid);
                    modCasList.add(tempList);
                    tempDecu = tempDecu + duration;
                }
            }
            basebean.writeLog("modCasList: " + modCasList);
            basebean.writeLog("tempDecu: " + tempDecu);

            if (modCasList != null && modCasList.size() > 0) {
                //修改转事假建模台账
                boolean b = rs.executeBatchSql(modLeaveSql, modCasList);
                basebean.writeLog("b: " + b);
                //修改转事假建模台账
                boolean b1 = rs.executeBatchSql(modCasSql, modCasList);
                basebean.writeLog("b1: " + b1);
                //修改加班抵扣转事假
                boolean b2 = rs.executeUpdate(modDecuSql, tempDecu, resourceId, kqDate.substring(0, 7));
                basebean.writeLog("b2: " + b2);
            }

        }

        return true;
    }

    public Map<String, Object> customLog( Map<String, String> params) {
        Map<String, Object> result = new HashMap<>();
        boolean flag = false;
        List<List> insertParams = new ArrayList<>();
        List insertPa = new ArrayList<>();

        String functionType = Util.null2String(params.get("functionType"));
        String changeBeforeData = Util.null2String(params.get("changeBeforeData"));
        String changeAfterData = Util.null2String(params.get("changeAfterData"));
        String changeDateTime = Util.null2String(params.get("changeDateTime"));
        if (StringUtils.isNotBlank(functionType) && StringUtils.isNotBlank(changeBeforeData)
                && StringUtils.isNotBlank(changeAfterData) && StringUtils.isNotBlank(changeDateTime)) {
            insertPa.add(functionType);
            insertPa.add(changeBeforeData);
            insertPa.add(changeAfterData);
            insertPa.add(changeDateTime);
            insertParams.add(insertPa);
        } else {
            result.put("res",flag);
            result.put("msg","入参存在空值，请检查参数");
            return result;
        }
        insertParams.add(insertPa);
        String insertSql = "insert into uf_KQCustomLog (functionType, changeBeforeData, changeAfterData, changeDate, " +
                " formmodeid, modedatacreater, modedatacreatertype, modedatacreatedate, modedatacreatetime ) values (?,?,?,?,?,?,?,?,?)";
        RecordSet rs = new RecordSet();
        flag = rs.executeBatchSql(insertSql, insertParams);
        if (flag) {
            result.put("res",flag);
            result.put("msg","保存成功！");
            return result;
        } else {
            result.put("res",flag);
            result.put("msg","保存失败！");
            return result;
        }
    }

    public boolean customLogBatch( List<Map<String, String>> params) {
        boolean result = false;
        List<List> insertParams = new ArrayList<>();
        List insertPa = new ArrayList<>();
        insertParams.add(insertPa);
        String  insertSql = "insert into uf_KQCustomLog () values ()";
        RecordSet rs = new RecordSet();
        result = rs.executeBatchSql(insertSql, insertParams);
        return result;
    }


}
